#! /bin/bash -

# 账号安全设置
# $1 登录失败次数上限
# $2 账号锁定时间 单位 秒
account_safed()
{
	realfile="";
	strfaillock1="auth		required  pam_tally2.so deny=$1 even_deny_root unlock_time=$2";
	strRegex="^auth.*pam_tally2.so.*";

#	if [ -e "/etc/pam.d/system-auth" ]; then
#		realfile="/etc/pam.d/system-auth";
#	elif [ -e "/etc/pam.d/common-auth" ]; then
#		realfile="/etc/pam.d/common-auth";
#	else
#		return;
#	fi	

	
	files="/etc/pam.d/system-auth /etc/pam.d/password-auth /etc/pam.d/common-auth"
	for realfile in ${files};
	do
		if [ ! -e "$realfile" ];then
			continue;
		fi

		if [ -L "$realfile" ];then
			realfile=`readlink -f  ${realfile}`;
		fi
		if grep -q -e "${strRegex}" ${realfile} ;then
			sed -i "/${strRegex}/d" ${realfile}
		fi
		sed -i "/^auth.*pam_deepin_auth.so.*/i ${strfaillock1}" ${realfile}
	done
}
# 密码复杂度设置
# $1 开关状态
passwd_complex()
{
	realfile="";
	strCracklib="password    requisite     pam_cracklib.so minclass=3 retry=3 reject_username gecoscheck";
	strRegex="^password.*pam_cracklib.so.*";

	files="/etc/pam.d/system-auth /etc/pam.d/password-auth /etc/pam.d/common-password"
	for realfile in ${files};
	do
		if [ ! -e "$realfile" ];then
			continue;
		fi
		
		if [ -L "$realfile" ];then
			realfile=`readlink -f  ${realfile}`;
		fi

		if [ "$1" = "0" ];then
			if grep -q -e "${strRegex}" ${realfile} ;then
				sed -i "/${strRegex}/d" ${realfile}
			fi
			return;
		fi
		
		if grep -q -e "${strRegex}" ${realfile} ;then
			sed -i "s/${strRegex}/${strCracklib}/g" ${realfile}
		elif grep -q -e "^password.*pam_pwquality.so.*" ${realfile}; then
			sed -i "s/^password.*pam_pwquality.so.*/${strCracklib}/g" ${realfile}
		else
			sed -i "/^password.*pam_unix.so.*/i ${strCracklib}" ${realfile}
		fi
	done
}
#修改已存在用户的密码使用期限
chage_pwd_maxdays()
{
	logindef="/etc/login.defs"
	passwdfile="/etc/passwd"
	if [ ! -e "${logindef}" -o ! -e "${passwdfile}" ];then
		return -1
	fi
	
	users=""
	maxDays=0
	minUid=0
	maxUid=60000
	maxDays=`grep -e "^PASS_MAX_DAYS" ${logindef}| awk '{print $2}'`;
	minUid=`grep -e "^UID_MIN" ${logindef}| awk '{print $2}'`;
	maxUid=`grep -e "^UID_MAX" ${logindef}| awk '{print $2}'`;
	if [ ${minUid} -lt 500 -o ${maxDays} -lt 0 ];then
		return -1;
	fi

	users=`awk -F ":" -v starUid=$minUid -v endUid=$maxUid '{if($3>=starUid && $3 <= endUid){print $1}}' ${passwdfile}`
	for user in $users
	do
		chage -M $maxDays $user
	done
}

# 密码安全设置
# $1 密码最小长度
# $2 历史密码不可重复限制
# $3 密码最长使用时间
password_safed()
{
	realfile="";
	strRegex="^password.*pam_unix.so.*";

	files="/etc/pam.d/system-auth /etc/pam.d/password-auth /etc/pam.d/common-password"
	for realfile in ${files};
	do
		if [ ! -e "$realfile" ];then
			continue;
		fi

		if [ -L "$realfile" ];then
			realfile=`readlink -f  ${realfile}`;
		fi
		strUnix=`grep -e "${strRegex}" ${realfile}`;
		if [ -z "$strUnix" ]; then
		 	return
		fi 

		if echo "$strUnix" | grep -q "minlen=" ; then
			strUnix=`echo "$strUnix"|sed -r "s/minlen=[[:digit:]]{1,}/minlen=${1}/g"` ;
		else
			strUnix=${strUnix}" minlen=${1}";
		fi

		if echo "$strUnix" | grep -q "remember=" ; then
			strUnix=`echo "$strUnix"|sed -r "s/remember=[[:digit:]]{1,}/remember=${2}/g"` ;
		else
			strUnix=${strUnix}" remember=${2}";
		fi

		sed -i "s/${strRegex}/${strUnix}/g" ${realfile};
	done

	#change /etc/login.defs
	sed -i "s/^PASS_MAX_DAYS.*/PASS_MAX_DAYS	${3}/g" /etc/login.defs;
	chage_pwd_maxdays
}

back_originconf()
{
	backpath=$1;
	if [ ! -e "$backpath" ];then
		realfile="";
		mkdir -p ${backpath}
		if [ -e "/etc/pam.d/system-auth" ]; then
			realfile="/etc/pam.d/system-auth";
			if [ -L "$realfile" ];then
				realfile=`readlink -f ${realfile}`;
			fi
			cp ${realfile} ${backpath}/
		fi
		if [ -e "/etc/pam.d/password-auth" ]; then
			realfile="/etc/pam.d/password-auth";
			if [ -L "$realfile" ];then
				realfile=`readlink -f ${realfile}`;
			fi
			cp ${realfile} ${backpath}/
		fi
		if [ -e "/etc/pam.d/common-password" ]; then
			realfile="/etc/pam.d/common-password";
			if [ -e "${realfile}" ];then
				cp ${realfile} ${backpath}/
			fi
			realfile="/etc/pam.d/common-auth";
			if [ -e "${realfile}" ];then
				cp ${realfile} ${backpath}/
			fi
		fi
	fi
}

#$1 :配置文件路径和文件名
#############################start run#############################################
#改功能暂时屏蔽
#exit 0

backpath="/var/.confback";
runpath=$(dirname $(readlink -f "$0"));

conf_file=""
if [ $# -ge 1 ]; then
	conf_file=$1;
fi
if [ "${conf_file}" = "" ] || [ ! -e "${conf_file}" ];then
	exit 0
fi
echo "conf_file=${conf_file}"

#策略关闭时，恢复所有的配置信息
switch_state=`grep -e "^SWITCH_ENABLE" ${conf_file}| awk -F '=' '{print $2}'`;
if [ "${switch_state}" = "0" ];then
	if [ -e "$backpath" ];then
		rm -fr /etc/passwd.lock
		cp -f ${backpath}/* /etc/pam.d/ 
		rm -fr $backpath
		sed -i "s/^PASS_MAX_DAYS.*/PASS_MAX_DAYS	99999/g" /etc/login.defs;
		chage_pwd_maxdays
		rm -fr /var/log/tallylog
	fi
	exit 0;
fi
#备份配置文件
back_originconf ${backpath} ${runpath}

#先删除该文件，不然密码安全无法生效
rm -fr /etc/passwd.lock

#密码安全设置
min_len=`grep -e "^PASSWD_MIN_LEN" ${conf_file}| awk -F '=' '{print $2}'`;
max_days=`grep -e "^PASSWD_MAX_DAYS" ${conf_file}| awk -F '=' '{print $2}'`;
remember=`grep -e "^PASSWD_REMEMBER" ${conf_file}| awk -F '=' '{print $2}'`;
echo "password_safed ${min_len} ${remember} ${max_days}"
password_safed ${min_len} ${remember} ${max_days}

#账号安全设置
login_cnt=`grep -e "^ACCOUNT_FAIL_LOGIN" ${conf_file}| awk -F '=' '{print $2}'`;
lock_time=`grep -e "^ACCOUNT_LOCK_TIME" ${conf_file}| awk -F '=' '{print $2}'`;
echo "account_safed ${login_cnt} ${lock_time}"
#account_safed ${login_cnt} ${lock_time}

#弱密码字典，密码复杂度设置
pass_weak=`grep -e "^CHECK_WEAK_PASSWD" ${conf_file}| awk -F '=' '{print $2}'`;
echo "pass_weak ${pass_weak}"
if [ "${pass_weak}" = "1" ];then
	if [ ! -d "/var/cache/cracklib" ];then
		mkdir /var/cache/cracklib
	fi
	passwd_complex ${pass_weak}
	custom_file=""
	pass_custom=`grep -e "^USE_CUSTOM_WEAK_PASSWD" ${conf_file}| awk -F '=' '{print $2}'`;
	echo "pass_custom ${pass_custom}"
	if [ "${pass_custom}" = "1" ];then
		custom_file=${runpath}/etc/360sd/custom.lst
	else
		custom_file=${runpath}/etc/360sd/password.lst
	fi
	if [ -e "/usr/sbin/create-cracklib-dict" ];then
		export PATH=$PATH:/usr/sbin/
	else
		export PATH=$PATH:/opt/360sdforcnos/crack/	
	fi
	create-cracklib-dict ${custom_file}
fi

#是否允许添加删除用户
account_switch=`grep -e "^ACCOUNT_MANAGE_ENABLE" ${conf_file}| awk -F '=' '{print $2}'`;
if [ "${account_switch}" = "1" ];then
	rm -fr /etc/passwd.lock
else
	touch /etc/passwd.lock
fi
